import React, {useState, useEffect} from 'react';

import {StyleSheet, Appearance, Dimensions, Platform} from 'react-native';
import {SafeAreaProvider, SafeAreaView} from 'react-native-safe-area-context';

import {useSelector, useDispatch} from 'react-redux';
import {RootState} from '@caj/redux/store';
import {appVarActions} from '@caj/configs/appVarReducer';

import imgSrc from '@caj/img/maimg.png';
import {placeholder} from '@caj/lang/default';
import {defaultHeaderStyle, themeNB, themeSelector} from '@caj/configs/colors';
import {saveVarChanges} from '@caj/helper/appData';

import {Box, Input, VStack, Center, Avatar, Text, Button} from 'native-base';
import {View} from 'react-native';

import {
  getFocusedRouteNameFromRoute,
  LinkingOptions,
  NavigationContainer,
  NavigatorScreenParams,
  ParamListBase,
  RouteProp,
  useNavigation,
  useRoute,
} from '@react-navigation/native';
import {
  createNativeStackNavigator,
  NativeStackNavigationOptions,
  NativeStackNavigationProp,
} from '@react-navigation/native-stack';
import {createBottomTabNavigator} from '@react-navigation/bottom-tabs';
import AccountTab, {
  AccountStackNavigatorParamList,
  AccountTabName,
} from './tabs/main/AccountTab';
import CalendarTab, {
  CalendarStackNavigatorParamList,
} from './tabs/main/CalendarTab';
import MapsTab, {MapsStackNavigatorParamList} from './tabs/main/MapsTab';
import ChatTab, {ChatStackNavigatorParamList} from './tabs/main/ChatTab';
import {
  FadeInView,
  SlideFromLeftView,
  SlideFromRightView,
} from './helper/animations';
import {
  LoginStackNavigatorParamList,
  RegisterScreenAnim,
} from './components/NotLoggedIn';
import ChatMainScreen from './components/chat/screen/main';
import ChatScreenStack from './components/chat/screen/main';
import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import {ThemeMode} from './configs/appVar';

const styles = StyleSheet.create({
  container: {
    height: '100%',
    flex: 1,
  },
});

export const linking: LinkingOptions<{}> = {
  prefixes: ['http://'],
};

export type RootStackNavigatorParamList = {
  Home: NavigatorScreenParams<HomeStackNavigatorParamList>;
  Register: NavigatorScreenParams<LoginStackNavigatorParamList>;
};

export type RootScreenNavigationProp =
  NativeStackNavigationProp<RootStackNavigatorParamList>;

export default function Navigation() {
  return (
    <Stack.Navigator
      screenOptions={{
        headerShown: false,
      }}>
      <Stack.Screen name="Home" component={HomeStack} />
      <Stack.Screen
        name="Register"
        component={RegisterScreenAnim}
        options={{
          animation: Platform.OS === 'ios' ? 'fade' : 'slide_from_left',
        }}
      />
    </Stack.Navigator>
  );
}

const Stack = createNativeStackNavigator<RootStackNavigatorParamList>();

export type HomeStackNavigatorParamList = {
  Account: NavigatorScreenParams<AccountStackNavigatorParamList>;
  Calendar: NavigatorScreenParams<CalendarStackNavigatorParamList>;
  Maps: NavigatorScreenParams<MapsStackNavigatorParamList>;
  ChatList: NavigatorScreenParams<ChatStackNavigatorParamList>;
};

export type HomeScreenNavigationProp =
  NativeStackNavigationProp<HomeStackNavigatorParamList>;

function AccountTabAnim(props: any) {
  return (
    <SlideFromLeftView>
      <AccountTab {...props} />
    </SlideFromLeftView>
  );
}

function CalendarTabAnim(props: any) {
  return (
    <FadeInView>
      <CalendarTab {...props} />
    </FadeInView>
  );
}
function MapsTabAnim(props: any) {
  return (
    <FadeInView>
      <MapsTab {...props} />
    </FadeInView>
  );
}
function ChatTabAnim(props: any) {
  return (
    <SlideFromLeftView>
      <ChatTab {...props} />
    </SlideFromLeftView>
  );
}

function HomeStack() {
  const getTabBarVisibility = (
    route: RouteProp<HomeStackNavigatorParamList, 'ChatList'>,
  ) => {
    const routeName = getFocusedRouteNameFromRoute(route);
    const hideOnScreens = ['Chat']; // put here name of screen where you want to hide tabBar
    if (routeName !== undefined) {
      return hideOnScreens.indexOf(routeName) <= -1;
    } else {
      return true;
    }
  };

  return (
    <Tab.Navigator screenOptions={{headerShown: false, unmountOnBlur: false}}>
      <Tab.Screen name="Account" component={AccountTabAnim} />
      <Tab.Screen name="Calendar" component={CalendarTabAnim} />
      <Tab.Screen name="Maps" component={MapsTabAnim} />
      <Tab.Screen
        name="ChatList"
        component={ChatTabAnim}
        options={({route}) => {
          return {
            tabBarStyle: {
              display: getTabBarVisibility(route) ? 'flex' : 'none',
            },
          };
        }}
      />
    </Tab.Navigator>
  );
}

const Tab = createBottomTabNavigator<HomeStackNavigatorParamList>();

export function WebBackButtonOptions<L extends ParamListBase>(
  nav: NativeStackNavigationProp<L>,
  _theme: ThemeMode,
  place?: 'registration',
): NativeStackNavigationOptions {
  if (Platform.OS === 'web') {
    const theme = themeNB(_theme);

    return {
      headerLeft: () => (
        <MaterialIcon
          style={{padding: 8, marginLeft: 8, marginRight: -8}}
          name="arrow-back"
          onPress={() => {
            nav.pop();
          }}
          size={24}
          color={defaultHeaderStyle(_theme, place).headerTintColor}
        />
      ),
    };
  }

  return {};
}
